# -*- coding: utf-8 -*-
from enum import Enum
from typing import Any

from thx.giocaonline.exceptions import ValidationError


class CodeEnum(Enum):
    @classmethod
    def get(cls, code: str):
        return getattr(cls, f"CODE_{code}").value


# REGIONI
class Region(Enum):
    """Tabella 8.4) di residenza del titolare del conto di gioco."""
    PIEMONTE = "01"
    VALLE_DAOSTA = "02"
    LOMBARDIA = "03"
    VENETO = "05"
    FRIULI_VENEZIA_GIULIA = "06"
    LIGURIA = "07"
    EMILIA_ROMAGNA = "08"
    TOSCANA = "09"
    UMBRIA = "10"
    MARCHE = "11"
    LAZIO = "12"
    ABRUZZO = "13"
    MOLISE = "14"
    CAMPANIA = "15"
    PUGLIA = "16"
    BASILICATA = "17"
    CALABRIA = "18"
    SICILIA = "19"
    SARDEGNA = "20"
    PROVINCIA_AUTONOMA_DI_BOLZANO = "21"
    PROVINCIA_AUTONOMA_DI_TRENTO = "22"
    ESTERO = "99"

    @classmethod
    def get(cls, region: str):
        region = region.upper().replace(" ", "_").replace("'", "")
        if 'BOLZANO' in region:
            region = "PROVINCIA_AUTONOMA_DI_BOLZANO"
        if 'TRENTO' in region:
            region = "PROVINCIA_AUTONOMA_DI_TRENTO"
        return getattr(cls, region).value


# FLAG DI RIACQUISTO
class FlagRiacquisto(Enum):
    """Impostare a 1 per un riacquisto, 2 per un add-on, 0 negli altri casi"""

    RIACQUISTO = 1
    ADD_ON = 2
    OTHER = 0


# Tabella dei tipi di terminali di accettazione gioco (TAG)
class CodeTipoTag(CodeEnum):
    """Codici dei tipi di terminali di accettazione gioco utilizzati dal giocatore."""

    CODE_4 = "Servizio telefonico automatico"
    CODE_5 = "Servizio telefonico con operatore"
    CODE_7 = "Canale Televisivo - TV Interattiva"

    CODE_10 = "Desktop tramite browser (Android/Unix like)"
    CODE_11 = "Desktop tramite browser (iOS)"
    CODE_12 = "Desktop tramite browser (Windows)"
    CODE_14 = "Desktop tramite browser (Altro SO)"

    CODE_20 = "Desktop tramite applicazione client (Android / Unix like)"
    CODE_21 = "Desktop tramite applicazione client (iOS)"
    CODE_22 = "Desktop tramite applicazione client (Windows)"
    CODE_24 = "Desktop tramite applicazione client (Altro SO)"

    CODE_30 = "Mobile Site, browser con sito ottimizzato per la fruizione da " \
              "dispositivo mobile con sistema operativo Android /Unix like"
    CODE_31 = "Mobile Site, browser con sito ottimizzato per la fruizione da " \
              "dispositivo mobile con sistema operativo iOS/Apple"
    CODE_32 = "Mobile Site, browser con sito ottimizzato per la fruizione da " \
              "dispositivo mobile con sistema operativo Windows/Microsoft"
    CODE_34 = "Mobile Site, browser con sito ottimizzato per la fruizione da " \
              "dispositivo mobile con altro sistema operativo"

    CODE_40 = "Applicazione client per Smartphone con sistema operativo Android/Unix like"
    CODE_41 = "Applicazione client per Smartphone con sistema operativo iOS/Apple"
    CODE_42 = "Applicazione client per Smartphone con sistema operativo Windows/Microsoft"
    CODE_43 = "Applicazione client per Smartphone con sistema operativo Blackberry"
    CODE_44 = "Applicazione client per Smartphone con altro sistema operativo"

    CODE_50 = "Applicazione client per Tablet con sistema operativo Android /Unix like"
    CODE_51 = "Applicazione client per Tablet con sistema operativo iOS/apple"
    CODE_52 = "Applicazione client per Tablet con sistema operativo Windows/Microsoft"
    CODE_53 = "Applicazione client per Tablet con sistema operativo Blackberry"
    CODE_54 = "Applicazione client per Tablet con altro sistema operativo"


# SESSION ATTRIBUTES
class SessionAttribute:
    @classmethod
    def validate(cls, value: Any) -> None:
        raise NotImplementedError("`validate` class method must be implemented")


class TPM(SessionAttribute, Enum):
    """Attributo di sessione: TMP."""

    P = "percentuale"
    S = "somma minima garantita"
    V = "montepremi variabile"

    @classmethod
    def validate(cls, value: str) -> None:
        try:
            getattr(cls, value)
        except AttributeError:
            raise ValidationError("Invalid TPM provided (%s)" % value)


class PRM(SessionAttribute):
    """Attributo di sessione: PRM.

    Indica la percentuale montepremi (obbligatorio). Per TPM=V indicare la percentuale media.
    """

    @classmethod
    def validate(cls, value: int) -> None:
        try:
            assert isinstance(value, int)
            assert value > 8000
        except AssertionError:
            raise ValidationError("Invalid PRM provided (%s)" % value)


class SGM(SessionAttribute):
    """Attributo di sessione: SGM.

    Qualsiasi importo: somma minima garantita (obbligatorio se TPM=S) (espressa in centesimi di euro)
    """

    @classmethod
    def validate(cls, value: float) -> None:
        try:
            assert isinstance(value, float)
            assert value >= 0
        except AssertionError:
            raise ValidationError("Invalid SGM provided (%s)" % value)


class RBY(SessionAttribute, Enum):
    """Attributo di sessione: RBY.

    Definisce la sessione in cui è consentito il riacquisto del biglietto (obbligatorio)
    """

    ZERO = "nessuno"
    ONE = "rebuy"
    TWO = "add-on"
    TREE = "entrambi"

    map_items = {
        "0": ZERO,
        "1": ONE,
        "2": TWO,
        "3": TREE,
    }

    def __getattr__(self, item):
        return super().__getattr__(self.map_items.get(str(item), item))

    @classmethod
    def validate(cls, value: str) -> None:
        try:
            getattr(cls, value)
        except AttributeError:
            raise ValidationError("Invalid RBY provided (%s)" % value)


class NMG(SessionAttribute):
    """Attributo di sessione: NMG.

    Indica il numero di giocatori previsti.
    """

    @classmethod
    def validate(cls, value: int) -> None:
        try:
            assert isinstance(value, int)
            assert value > 1
        except AssertionError:
            raise ValidationError("Invalid NMG provided (%s)" % value)


class BON(SessionAttribute, Enum):
    """Attributo di sessione: BON.

    Indica una sessione che ammette di essere giocata con bonus.

    Non obbligatorio, deve essere valorizzato solo nel caso in cui la sessione di gioco permette l’utilizzo del bonus.

    Il valore del bonus può essere diverso per ogni concessionario partecipante al circuito;
    pertanto, il concessionario proponente indicherà nel messaggio di apertura che la
    sessione prevede il BONUS e il valore dell’importo nominale del diritto, nei messaggi di
    acquisto dei diritti dei propri clienti il concessionario, indicherà l’importo del diritto e
    quello dell’eventuale bonus
    """

    B = "sessione che ammette di essere giocata con bonus"

    @classmethod
    def validate(cls, value: str) -> None:
        try:
            getattr(cls, value)
        except AttributeError:
            raise ValidationError("Invalid BON provided (%s)" % value)


class VIN(SessionAttribute, Enum):
    """Attributo di sessione: VIN.

    Sessione con vincite vincolate.

    L’attributo VIN deve essere impostato qualora il concessionario apre la sessione di gioco
    prevedendo l’utilizzo delle vincite in essa ottenute, per l’acquisto del diritto di partecipazione
    in una sessione di gioco futura.
    """

    V = "sessione con vincite vincolate"

    @classmethod
    def validate(cls, value: str) -> None:
        try:
            getattr(cls, value)
        except AttributeError:
            raise ValidationError("Invalid VIN provided (%s)" % value)


class JK1(SessionAttribute):
    """Attributo di sessione: JK1.

    Indica una sessione con jackpot interno al gioco; vale 1 qualora sia previsto tale tipologia di Jackpot.

    L’attributo JK1 deve essere impostato qualora il concessionario apre la sessione di gioco
    che prevede erogazione di jackpot, derivanti dalla matematica del gioco, che
    incrementano la vincita spettante al giocatore.
    """

    @classmethod
    def validate(cls, value: int) -> None:
        try:
            assert value in (0, 1)
        except AssertionError:
            raise ValidationError("Invalid JK1 provided (%s)" % value)


class JK2(SessionAttribute):
    """Attributo di sessione: JK2.

    Indica una sessione con jackpot aggiuntivi; vale 1 qualora sia previsto tale tipologia di Jackpot.

    L’attributo JK2 deve essere impostato qualora il concessionario apre la sessione di gioco
    che prevede erogazione di jackpot aggiuntivi, assegnati indipendentemente dal gioco e
    che si aggiungono alla percentuale di RTP per la determinazione del margine per il concessionario.
    """

    @classmethod
    def validate(cls, value: int) -> None:
        try:
            assert value in (0, 1)
        except AssertionError:
            raise ValidationError("Invalid JK2 provided (%s)" % value)


class CUP(SessionAttribute):
    """Attributo di sessione: CUP.

    Sessione di gioco in modalità CUP
        0 = (valore di default) no
        1 = si
    """

    @classmethod
    def validate(cls, value: int) -> None:
        try:
            assert value in (0, 1)
        except AssertionError:
            raise ValidationError("Invalid CUP provided (%s)" % value)


class NSI(SessionAttribute):
    """Attributo di sessione: NSI.

    Numero delle sezioni del primo livello di un torneo multifase (numero delle sezioni ingresso). Specifica
    quante sono le sezioni di primo livello del torneo multifase in cui è consentito l’acquisto del diritto di
    partecipazione. Da 1 a massimo 16.
    """

    @classmethod
    def validate(cls, value: int) -> None:
        try:
            assert isinstance(value, int)
            assert 1 <= value <= 16
        except AssertionError:
            raise ValidationError("Invalid CUP provided (%s)" % value)


class SessionAttributes(Enum):
    """Attributi che caratterizzano una sessione di gioco"""

    TPM = TPM
    PRM = PRM
    SGM = SGM
    RBY = RBY
    NMG = NMG
    BON = BON
    VIN = VIN
    JK1 = JK1
    JK2 = JK2
    CUP = CUP
    NSI = NSI

    @classmethod
    def validate(cls, key, value):
        try:
            sa = getattr(cls, key)
        except AttributeError:
            raise ValidationError("Invalid session attribute (%s) provided." % key)

        sa.validate(value)

        if isinstance(value, float):
            # Qualora i valori siano di tipo numerico essi dovranno essere inseriti comprensivi di due cifre decimali
            if len(str(value).split('.')[-1]) > 2:
                raise ValidationError("float value (%d) must have less than 3 decimal places." % value)

        return value


# Tabella dei codici rete
class CodeRete(Enum):
    """Codici identificativi della rete da utilizzare nel messaggio 220/420/620, 280/430/630, 340, 510."""

    GIOCHI_PUBBLICI_SPORT = 2
    # CONCESSIONARI BERSANI SPORTIVI ex art. 38, comma 2 D.L. 223/2006

    GIOCHI_PUBBLICI_IPPICA = 3
    # COMPRENDE:
    # - CONCESSIONARI BERSANI IPPICI ex art. 38, comma 4 D.L. 223/2006
    # - CONCESSIONARI IPPICI D.L. 149/08

    RINNOVATO_SCOMMESSE_IPPICHE = 7
    # COMPRENDE:
    # - AGENZIE SCOMMESSE IPPICHE
    # - IPPODROMI

    RINNOVATO_SCOMMESSE_SPORTIVE = 8
    # AGENZIE SCOMMESSE SPORTIVE

    SUPERENALOTTO = 12
    BINGO = 13
    GAD = 14
    # CONCESSIONARI di cui all’art. 24 della legge n. 88 del 7 luglio 2009


# PLAY TYPE
class PlayType(CodeEnum):
    # art.  1  comma  1  del  decreto  di disciplina del gioco
    CODE_1 = "Giochi di abilità"
    # art. 1 comma 2 let. a) del decreto di disciplina del gioco
    CODE_2 = "Giochi di sorte a quota fissa"
    CODE_3 = "Giochi di sorte a quota fissa tipo `istantaneo`"
    # art. 1 comma 2 let. b) del decreto di disciplina del gioco
    CODE_4 = "Giochi di carte organizzati in forma diversa dal torneo – gioco tra giocatori"
    CODE_5 = "Giochi di carte organizzati in forma diversa dal torneo  – gioco in solitario"


# ADDRESSES
class TestAddress(CodeEnum):
    CODE_1 = "http://10.70.78.131/GiochiDiAbilitaV2_1/ServletFactoryFirma_V213_TR"
    CODE_245 = "http://10.70.78.131/GiochiDiAbilitaV2_1/ServletFactoryFirma_V213_QF"
    CODE_24M580 = "http://10.70.78.131/GiochiDiAbilitaV2_1/ServletFactoryFirma_V213_580"
    CODE_3 = "http://10.70.78.131/GiochiDiAbilitaV2_1/ServletFactoryFirma_V213_SH"
    CODE_3M780 = "http://10.70.78.131/GiochiDiAbilitaV2_1/ServletFactoryFirma_V213_780"


class ProdAddress(CodeEnum):
    # TODO
    CODE_1 = ""
    CODE_245 = ""
    CODE_24M580 = ""
    CODE_3 = ""
    CODE_3M780 = ""


# GIOCA ONLINE SPECIFIC
class PrizePoolType(Enum):
    PERCENTAGE = "PERCENTUALE"
    VARIABLE = "VARIABILE"
    MINIMUM_GUARANTEED_SUM = "SOMMA_MIN_GARANTITA"


class GiocaOnlineErrorTypes(Enum):
    OK = 0
    OPERATOR_ERROR = 1
    AAMS_ERROR = 2
    GOL_ERROR_3 = 3  # impossibile verificare esito richiesta
    GOL_ERROR_4 = 4  # impossibile inviare richiesta all'operatore
    GOL_ERROR_5 = 5  # impossibile reperire risposta (richiesta mai inviata)
    GOL_ERROR_6 = 6  # impossibile reperire risposta (richiesta pending)
    GOL_ERROR_7 = 7  # impossibile reperire risposta (richiesta completata)
    GOL_ERROR_8 = 8  # errore sui sistemi dell'operatore (non decodificabile)
    GOAL_ERROR_1013 = 1013

    def get_description(self):
        if self == GiocaOnlineErrorTypes.OK:
            return "ok"
        if self == GiocaOnlineErrorTypes.OPERATOR_ERROR:
            return "errore sui sistemi dell'operatore"
        if self == GiocaOnlineErrorTypes.AAMS_ERROR:
            return "errore AAMS"
        if self == GiocaOnlineErrorTypes.GOL_ERROR_3:
            return "impossibile verificare esito richiesta"
        if self == GiocaOnlineErrorTypes.GOL_ERROR_4:
            return "impossibile inviare richiesta all'operatore"
        if self == GiocaOnlineErrorTypes.GOL_ERROR_5:
            return "impossibile reperire risposta (richiesta mai inviata)"
        if self == GiocaOnlineErrorTypes.GOL_ERROR_6:
            return "impossibile reperire risposta (richiesta pending)"
        if self == GiocaOnlineErrorTypes.GOL_ERROR_7:
            return "impossibile reperire risposta (richiesta completata)"
        if self == GiocaOnlineErrorTypes.GOL_ERROR_8:
            return "errore sui sistemi dell'operatore (non decodificabile)"
        if self == GiocaOnlineErrorTypes.GOAL_ERROR_1013:
            return "errore sui sistemi dei concessionari. La risposta non è pervenuta in tempi accettabili."


class Rebuy(Enum):
    NONE = "NESSUNO"
    REBUY = "REBUY"
